

function  [BC, EC, Mn, N0, Rtotal] = Drake_model(K_r,K_ep,CN_s,Pulse_C,CN_enz,CN_m,CN_pulse,temperature)
 
% This code version is meant to do a spin-up run of 100000 timesteps to reach
% a stable state and then deliver a pulse of DOC and DON of user-defined
% magnitude, and to follow the pulse for another period.;


T=11000;

dt = 0.1;

%Variables.;
BC = zeros(T,1);%Microbial biomass C;
EC = zeros(T,1);%Enzyme C;
%EN = zeros(T,1);%Enzyme N;
BN = zeros(T,1);%Microbial biomass N;
N0 = zeros(T,1);% mineral N;
DOC = zeros(T,1);
DON = zeros(T,1);
Rtotal = zeros(T,1);%total respiration;


%these variables will be used to evaluate the effect of a DOC/DON pulse;
%These are evaluated later in the code as running sums after the 
%pulse flag is tripped to 1;
DC_pulse = zeros(T,1);%C decomp following pulse
DN_pulse = zeros(T,1);%N decomp following pulse
Mn_pulse = zeros(T,1);%Net N min following pulse
Flag_pulse = zeros(1,1);

Mn = zeros(T,1);%Rate of N mineralization;
Ro = zeros(T,1);%C mineralized from overflow metabolism;
Jn = zeros(T,1);%Rate of inoranic N uptake;
EP_c = zeros(T,1);%Rate of enzyme production;
EL_c = zeros(T,1);%Rate of enzyme loss;
U_c = zeros(T,1);%Rate of microbial uptake of DOC;
U_n = zeros(T,1);%Rate of microbial uptake of DON;
D_n = zeros(T,1);%Rate of SON decomposition;

%Initial Conditions.;
BC(1) = 27;
EC(1) = 1.6;
EN(1) = 0.5;
BN(1) = 10;
N0(1) = 20;
%RootB(1)= 250;
U_c(1) = 0;

DC_pulse(1) = 0;%Sets the initial conditions of the "extra" C N, and netNmin pulses;
DN_pulse(1) = 0;
Mn_pulse(1) = 0;
Flag_pulse = 0;

%Parameters
%K_e = 0.05;%fraction of DOC uptake allocation to enzyme production in original Shimel model;
%K_ep = 0.0005; %parameter relating enzyme production to BC. 0.000005 hr-1 in Allison 2010, equals 0.00012 day-1;
K_l = 0.05;%enzyme decay constants. 
K_m = 0.01;%microbial maintenence rate, includes maintaining living cells and biomass turnover;
SUE = 0.5;%fraction of DOC uptake that is converted to biomass or enzymes (remainder is respired);
K_t = 0.012;%proportion of microbial biomass that dies each time-step. How does this differ from K_m?;
%K_r = 0.85;%proportion of dead microbial biomass that is recycled (available to microbes);
%CN_s = 27.6;%C:N ratio of the soil.;
%CN_m = 5; %7.16;%C:N ratio of the microbes.;
%CN_pulse=1000;
%CN_enz=3;%C:N ratio of the enzyme;

kappa_d = 1;
Vmaxuptake0 = 153846154; %pre-exponential rate of DOC uptake in mg DOC g-1 hr-1;
                            %obtained by dividing 100000000 mg DOC cm-3
                            %hr-1 by a bulk density of 0.65 g cm-3;
Eauptake = 47;%kJ mol-1 degree-1. Activation energy of DOC uptake;
gasconst = 0.008314;%kJ mol-1 k-1;
Kmuptakeslope = 0.015; %slope by which the km of DOC uptake increases with each increase in temperature;
                        %units of mg g-1 degree-1;
Kmuptake0 = 0.154; %Km of DOC uptake at 0 deg C, in mg g-1;

  for i=1:T
        
      %This code delivers a user-defined amount of DOC and DON at a set 
      %time-step and sets a flag "high". This flag triggers the running sums to
      %be evaluated in an if-then statement near the end of the main
      %for-loop.
      if i ==10000,
          DOC(i) = DOC(i) + Pulse_C;
          DON(i) = DON(i) + Pulse_C/CN_pulse;
          Flag_pulse = 1;
      end;
      

      
      Vmaxuptake = Vmaxuptake0*exp(-1*Eauptake./(gasconst*(temperature+273)));
      Kmuptake = Kmuptakeslope*temperature+Kmuptake0;
      
      
      Uc = Vmaxuptake*BC(i)*(DOC(i)/(Kmuptake + DOC(i))); %MM kinetics for DOC up;
      if Uc > DOC(i), Uc = DOC(i); %DOC uptake cannot exceed supply.;
      end;
      
      EL_c = K_l * EC(i);
      R_m  = K_m * BC(i);
      EP_c = K_ep * BC(i)*2;%K_ep has units of biomass, so *2 to change to C;
      R_e  = EP_c * (1-SUE)/SUE;
      
      
      Dn = Dc(EC(i),kappa_d) / CN_s;
      
%     
      Un = Vmaxuptake*BC(i)*(DON(i)/(Kmuptake + DON(i))); %MM kinetics for DOC up;
      if Un > DON(i), Un = DON(i); %DOC uptake cannot exceed supply.;
      end;
      
      CY_c = K_t*K_r*BC(i);
      H_c  = K_t*(1-K_r)*BC(i);
      
      EP_n = EP_c/CN_enz;
      CY_n = CY_c/CN_m;
      H_n  = H_c / CN_m;
      
      %J_n
      %if clim(Uc, R_m, EP_c, SUE, Un, CN_m)=='C',
      %    J_n = ( Uc - R_m - EP_c/SUE )*SUE/CN_m + EP_n - Un;
      %else
      %    J_n = 0;
      %end
      if Uc < R_m + EP_c/SUE + (Un - EP_n)*CN_m/SUE,
%           J_n = ( Uc - R_m - EP_c/SUE )*SUE/CN_m + EP_n - Un;
            J_n = 0;
      else
            J_n = ( Uc - R_m - EP_c/SUE )*SUE/CN_m + EP_n - Un;
            if J_n > Mn(i),
                  J_n = Mn(i);
            end;
            if J_n < 0,
                  J_n = 0;
            end;
%           J_n = 0;
      end
      
      %R_g
      if clim(Uc, R_m, EP_c, EP_n, SUE, Un, CN_m)=='C',
      %if Uc < R_m + EP_c/SUE + (Un + J_n - EP_n)*CN_m/SUE,
          R_g = (Uc - EP_c/SUE - R_m)*(1-SUE);
      else
          R_g = (Un + J_n - EP_n)*CN_m*(1-SUE)/SUE;
      end
      
      %R_o
      if clim(Uc, R_m, EP_c, EP_n, SUE, Un, CN_m)=='C',
      %if Uc < R_m + EP_c/SUE + (Un + J_n - EP_n)*CN_m/SUE,
          R_o = 0;
      else
          R_o = Uc - R_m - EP_c/SUE - (Un + J_n - EP_n)*CN_m/SUE;
      end
      
      %M_n
      if clim(Uc, R_m, EP_c, EP_n, SUE, Un, CN_m)=='C',
      %if Uc < R_m + EP_c/SUE + (Un + N0(i) - EP_n)*CN_m/SUE,
          M_n = Un - EP_n - ( Uc - R_m - EP_c/SUE )*SUE/CN_m;
      else
          M_n = 0;
      end
      
      Rtotal(i+1) = R_e + R_m + R_g;%sums total respiration;
      
      BC(i+1) = BC(i) + dt*( Uc - CY_c - EP_c - R_o - H_c - R_e - R_m - R_g );
      EC(i+1) = EC(i) + dt*( EP_c - EL_c );
      
      BN(i+1) = BN(i) + dt*( Un - CY_n - EP_n - M_n + J_n - H_n);
      %EN(i+1) = EN(i) + dt*( EP_n - EL_n );
      
      N0(i+1) = 0.6*N0(i) + dt*( M_n - J_n);
      DOC(i+1) = DOC(i) + dt*( Dc(EC(i),kappa_d) - Uc + CY_c);
      DON(i+1) = DON(i) + dt*( Dn        - Un + CY_n);
            
      Mn(i+1) = M_n;
      Jn(i+1) = J_n;
      Ro(i+1) = R_o;
      EP_c(i+1) = EP_c;
      EL_c(i+1) = EL_c;
      U_c(i+1) = Uc;
      U_n(i+1) = Un;
      D_n(i+1) = Dn;


      
      % After the pulse of DOC and DON, the flag is set "high" to 1,
      % triggering the following variables to be computed as running sums.
      if Flag_pulse == 1,
          DC_pulse(i+1) = DC_pulse(i) + Dc(EC(i),kappa_d);% DOC(i); %+ dt*( Dc(EC(i),kappa_d) - Uc + CY_c);%C decomp following pulse;
          DN_pulse(i+1) = DN_pulse(i) + Dn; %+ dt*( Dn        - Un + CY_n);%N decomp following pulse;
          Mn_pulse(i+1) = Mn_pulse(i) + M_n;
      else 
          DC_pulse(i) = 0;
          DN_pulse(i) = 0;
          Mn_pulse(i) = 0;
      end;
           
  end

end
    
function res = Dc(Enz,kappa_d)

  %kappa_d = 1;
    SOC = 1;
    K_es = 0.3;
    res = kappa_d * Enz / (K_es + Enz)*SOC;
end

%The following tests to see if the the microbes are C or N limited. 
%A text value of "C" or "N" is returned.
function lim = clim(Uc, R_m, EP_c, EP_n, SUE, Un, CN_m)
    if Uc < R_m + EP_c/SUE + (Un - EP_n)*CN_m/SUE,
        lim = 'C';
    else
        lim = 'N';
    end
end